<?php

class UsersModel
{
    /**
     * Every model needs a database connection, passed to the model
     * @param object $db A PDO database connection
     */
    function __construct($db) {
        try {
            $this->db = $db;
        } catch (PDOException $e) {
            exit('Database connection could not be established.');
        }
    }

    /**
     * Get all users from database
     */
    public function listUsers()
    {
        try {
            $table = 'user';
            $primaryKey = 'id';
            
            $columns = array(
                array( 'db' => 'id', 'dt' => 0 ),
                array( 'db' => 'username', 'dt' => 1 ),
                array( 'db' => 'user_type',  'dt' => 2 )
            );
            
            $sql_details = array(
                'user' => DB_USER,
                'pass' => DB_PASS,
                'db'   => DB_NAME,
                'host' => DB_HOST
            );
            
            require_once('application/libs/ssp.class.php');
            
            return json_encode(
                SSP::complex( $_GET, $sql_details, $table, $primaryKey, $columns, null, "username != 'admin'" )
            );
        } catch (Throwable $e) {
            throw new Exception($e);
        }
    }

    /**
     * Add a user to database
     * @param string $username username
     * @param string $password password
     * @param string $userType user_type
     */
    public function addUser($username, $password, $confirmPassword, $userType)
    {
        try {
            require_once('application/libs/utilities.php');

            // Clean the input from javascript
            $username = str_replace(' ', '', $username);
            $username = preg_replace('/\s+/', '', $username);
            $username = Utilities::cleanInput($username);

            $password = Utilities::cleanInput($password);
            $confirm_password = Utilities::cleanInput($password);
            $userType = Utilities::cleanInput($userType);
            //END Clean the input from javascript

            //Server-side validation
            if($this->checkUsername($username) > 0) 
            {
                throw new Exception('USER_ERR-200: Username already exist.');
            }

            if($password != $confirmPassword) 
            {
                throw new Exception('USER_ERR-202: Confirm password did not match.');
            }

            if(strlen($username)<6) 
            {
                throw new Exception('USER_ERR-203: Your username must consist of at least 6 characters.');
            }

            if(strlen($password)<5) 
            {
                throw new Exception('USER_ERR-204: Your password must consist of at least 5 characters.');
            }
            //END Server-side validation

            $password = password_hash($password, PASSWORD_DEFAULT);

            $sql = "INSERT INTO user (id, username, password, user_type) VALUES (UUID(), :username, :password, :userType)";
            
            $query = $this->db->prepare($sql);
            $query->execute(array(':username' => $username, ':password' => $password, ':userType' => $userType));

            if (!$query->rowCount())
            {
                throw new Exception('USER_ERR-201: User not saved.');
            }

        } catch (Throwable $e) {
            throw new Exception($e);
        }
    }

    /**
     * Reset user password
     * @param string $id id
     * @param string $password password
     */
    public function resetPassword($id, $password, $confirmPassword)
    {
        try {
            require_once('application/libs/utilities.php');

            // Clean the input from javascript
            $id = Utilities::cleanInput($id);
            $password = Utilities::cleanInput($password);
            $confirmPassword = Utilities::cleanInput($confirmPassword);
            // END Clean the input from javascript

            //Server side validation
            if($password != $confirmPassword) 
            {
                throw new Exception('USER_ERR-202: Confirm password did not match.');
            }

            if(strlen($password)<5) 
            {
                throw new Exception('USER_ERR-204: Your password must consist of at least 5 characters.');
            }
            //END Server side validation

            $password = password_hash($password, PASSWORD_DEFAULT);

            $sql = "UPDATE user SET password = :password WHERE id = :id";

            $query = $this->db->prepare($sql);
            $query->execute(array(':password' => $password, ':id' => $id));

            if (!$query->rowCount())
            {
                throw new Exception('USER_ERR-205: Password not reset.');
            }
        } catch (Throwable  $e) {
            throw new Exception($e);
        }
    }

    /**
     * Delete a user in the database
     * @param int $id Id of user
     */
    public function deleteUser($id)
    {
        require_once('application/libs/utilities.php');

        // clean the input from javascript
        $id = Utilities::cleanInput($id);

        $sql = "DELETE FROM user WHERE id = :id";
        $query = $this->db->prepare($sql);
        $query->execute(array(':id' => $id));

        if (!$query->rowCount())
        {
            throw new Exception('USER_ERR-206: User was not deleted.');
        }
    }

    /**
     * Check username if exist
     * @param string $username
     */
    public function checkUsername($username)
    {
        // clean the input from javascript
        $username = Utilities::cleanInput($username);

        $sql = "SELECT id FROM user WHERE username = :username";
        $query = $this->db->prepare($sql);
        $query->execute(array(':username' => $username));

        return $query->rowCount();
    }
}
